/*
 * Copyright (c) 2022-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

import 'package:target_framework/target_framework.dart';

/// The base class for all Target controllers.
///
/// A controller is a class which handles an incoming HTTP request by path.
abstract class TargetBaseController {
  /// A list of [TargetMiddleware] applied to this controller.
  List<TargetMiddleware> get middleware => [];

  /// The path of this controller.
  ///
  /// The path can either be a static path (e.g. `/users`) or contain one
  /// or more dybnamic path segments (e.g. `/users/<id>`). Parameters can be
  /// retrived via the `request.params` map.
  ///
  /// Alternativly the path can be a regular expression such as `/users/<id|[0-9]+>`,
  /// which would match any number value.  The regular expression will also handle
  /// a 'splat' route, catching any number of parameters (e.g. `/users/<splat|.*>`
  /// will handle `/users/123`, `/users/123/abc` and so on). The parameter value
  /// will be the caught splat path (e.g. `/users/123/abc` would result in `123/abc`).
  String get path;
}
