/*
 * Copyright (c) 2022-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

/// An exception representing that an incoming request was not handled
/// by a controller method.
class RequestNotHandled implements Exception {}

/// An exception representing that an incoming WebSocket handshake was aborted.
///
/// The reason for aborting is that usually the client's WebSocket connection
/// was not accepted by the server, for example an invalid origin or missing
/// headers.
class WebSocketHandshakeAborted implements Exception {
  /// Creates a new [WebSocketHandshakeAborted] instance.
  WebSocketHandshakeAborted(this.reason);

  /// The reason the handshake was aborted.
  final String reason;
}
