/*
 * Copyright (c) 2022-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

import 'package:target_framework/src/logger.dart';
import 'package:shelf/shelf.dart' as shelf;

/// Log the request information, such as the status code, the request method,
/// the time it took to process the request, and the the time of the request.
shelf.Middleware logRequest() {
  return (innerHandler) {
    return (request) async {
      final start = DateTime.now();
      final watch = Stopwatch()..start();

      return Future.sync(() => innerHandler(request)).then((response) {
        final elapsed = watch.elapsed;
        final method = request.method.toUpperCase();

        log.info(
          '[⏱ ] ${response.statusCode} ${method} ${_url(request)} (${elapsed.inMilliseconds}ms) ${_requestTime(start).padLeft(10)}',
        );

        return response;
      });
    };
  };
}

String _url(shelf.Request request) {
  return '/${request.url}';
}

String _requestTime(DateTime time) {
  return '${time.year}-${time.month}-${time.day} ${time.hour}:${time.minute}:${time.second}';
}
