/*
 * Copyright (c) 2022-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

import 'dart:convert' show Encoding;

import 'package:shelf/shelf.dart' as shelf show Request;
import 'package:stream_channel/stream_channel.dart';

/// An internal view implememntation of [Request].
///
/// This should not be used directly, and instead extended to overide
/// specific methods of the Shelf Request.
class ShelfRequestView implements shelf.Request {
  ShelfRequestView(this.delegate);

  shelf.Request delegate;

  @override
  bool get canHijack => delegate.canHijack;

  @override
  shelf.Request change(
      {Map<String, Object?>? headers,
      Map<String, Object?>? context,
      String? path,
      body}) {
    return delegate.change(
        headers: headers, context: context, path: path, body: body);
  }

  @override
  int? get contentLength => delegate.contentLength;

  @override
  Map<String, Object> get context => delegate.context;

  @override
  Encoding? get encoding => delegate.encoding;

  @override
  String get handlerPath => delegate.handlerPath;

  @override
  Map<String, String> get headers => delegate.headers;

  @override
  Map<String, List<String>> get headersAll => delegate.headersAll;

  @override
  Never hijack(void Function(StreamChannel<List<int>> p1) callback) {
    return delegate.hijack(callback);
  }

  @override
  DateTime? get ifModifiedSince => delegate.ifModifiedSince;

  @override
  bool get isEmpty => delegate.isEmpty;

  @override
  String get method => delegate.method;

  @override
  String? get mimeType => delegate.mimeType;

  @override
  String get protocolVersion => delegate.protocolVersion;

  @override
  Stream<List<int>> read() {
    return delegate.read();
  }

  @override
  Future<String> readAsString([Encoding? encoding]) {
    return delegate.readAsString(encoding);
  }

  @override
  Uri get requestedUri => delegate.requestedUri;

  @override
  Uri get url => delegate.url;
}
