/*
 * Copyright (c) 2022-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

import 'package:logging/logging.dart' as logging;

final log = logging.Logger('target');

class LogLevel {
  /// Name of this logging level.
  final String name;

  /// Unique value for this level. Used to order levels, so filtering can
  /// exclude messages whose level is under certain value.
  final int value;

  const LogLevel(this.name, this.value);

  /// Special key to turn on logging for all levels.
  static const LogLevel all = LogLevel('ALL', 0);

  /// Special key to turn off all logging.
  static const LogLevel off = LogLevel('OFF', 2000);

  /// Key for highly detailed tracing.
  static const LogLevel finest = LogLevel('FINEST', 300);

  /// Key for fairly detailed tracing.
  static const LogLevel finer = LogLevel('FINER', 400);

  /// Key for tracing information.
  static const LogLevel fine = LogLevel('FINE', 500);

  /// Key for static configuration messages.
  static const LogLevel config = LogLevel('CONFIG', 700);

  /// Key for informational messages.
  static const LogLevel info = LogLevel('INFO', 800);

  /// Key for potential problems.
  static const LogLevel warning = LogLevel('WARNING', 900);

  /// Key for serious failures.
  static const LogLevel severe = LogLevel('SEVERE', 1000);

  /// Key for extra debugging loudness.
  static const LogLevel shout = LogLevel('SHOUT', 1200);

  /// Returns as an instance of [Level] from the `logging` package.
  logging.Level get level {
    return logging.Level(name, value);
  }
}
