import 'package:target_framework/target_framework.dart';
import 'package:spec/spec.dart';

void main() {
  group('$FormData', () {
    test('it constructs with no form data', () {
      final f = FormData();
      expect(f.keys.length).toEqual(0);
    });

    test('it accepts initial form data', () {
      final f = FormData({
        'foo': [],
      });

      expect(f.keys.length).toEqual(1);
    });

    test('it returns the keys', () {
      final f = FormData({
        'foo': [],
        'bar': [],
      });

      expect(f.keys.toList()).toEqual(['foo', 'bar']);
    });

    test('it returns the values', () {
      final f = FormData({
        'foo': [],
        'bar': [FormDataEntryValue.string('foo')],
      });

      expect(f.values.toList()[0]).toEqual([]);
      expect(f.values.toList()[1]).toEqual([FormDataEntryValue.string('foo')]);
    });

    test('it overrides the get operator', () {
      final f = FormData({
        'foo': [],
        'bar': [FormDataEntryValue.string('foo')],
      });

      expect(f['foo']).toEqual([]);
      expect(f['bar']).toEqual([FormDataEntryValue.string('foo')]);
    });

    test('it accepts valid $FormDataEntryValue fields', () {
      final f = FormData({
        'foo': [
          FormDataEntryValue.string('foo'),
          FormDataEntryValue.string('bar'),
        ],
      });

      expect(f['foo']).toEqual([
        FormDataEntryValue.string('foo'),
        FormDataEntryValue.string('bar'),
      ]);
    });
  });
}
