import 'package:http_parser/http_parser.dart';
import 'package:spec/spec.dart';
import 'package:target_framework/target_framework.dart';
import 'package:http/http.dart' as http;
import 'package:path/path.dart' as path;
import 'utils.dart';

final int port = 3003;
void main() {
  group('FormData', () {
    test('should return empty form data ', () async {
      final controller = ManualTestController('/foo', postHandler: true);
      await startServer(port, controllers: [controller]);

      request(
        HttpMethod.post,
        '/foo',
      );

      final req = await controller.postHandlerCompleter!.future;

      final form = await req.formData();
      expect(form.keys.length).toEqual(0);
    });

    test('should return form data for www-url-encoded forms ', () async {
      final controller = ManualTestController('/foo', postHandler: true);
      await startServer(port, controllers: [controller]);

      request(
        HttpMethod.post,
        '/foo',
        body: 'foo=bar&bar=baz&foo=123',
        headers: Headers()
          ..append('content-type', 'application/x-www-form-urlencoded'),
      );

      final req = await controller.postHandlerCompleter!.future;

      final form = await req.formData();
      expect(form.keys.length).toEqual(2);

      final foo = form['foo'];
      expect(foo == null).isFalse();
      expect(foo!.length).toEqual(2);
      expect(foo[0]).toEqual(FormDataEntryValue.string('bar'));
      expect(foo[1]).toEqual(FormDataEntryValue.string('123'));

      final bar = form['bar'];
      expect(bar == null).isFalse();
      expect(bar!.length).toEqual(1);
      expect(bar[0]).toEqual(FormDataEntryValue.string('baz'));
    });

    test('should valid form data for form-data requests ', () async {
      final controller = ManualTestController('/foo', postHandler: true);
      await startServer(port, controllers: [controller]);

      final url = Uri.http('${server!.address.host}:${server!.port}', '/foo');

      final request = http.MultipartRequest('POST', url)
        ..fields['foo'] = 'bar'
        ..files.add(await http.MultipartFile.fromPath(
            'file', path.join(path.current, 'test', 'test_file.txt'),
            contentType: MediaType('text', 'plain')));

      request.send();

      final req = await controller.postHandlerCompleter!.future;

      final form = await req.formData();
      expect(form.keys.length).toEqual(2);

      final foo = form['foo'];
      expect(foo == null).isFalse();
      expect(foo!.length).toEqual(1);
      expect(foo[0]).toEqual(FormDataEntryValue.string('bar'));

      final file = form['file'];
      expect(file == null).isFalse();
      expect(file!.length).toEqual(1);

      file[0].map(
        string: (value) => Exception('Should not be a string'),
        file: (file) async {
          expect(file.name).toEqual('test_file.txt');
          await file
              .text()
              .then((value) => expect(value).toEqual('hello world!'));
        },
      );
    });
  });
}
