import 'package:spec/spec.dart';
import 'package:target_framework/src/view.dart';
import 'utils.dart';

// TODO not really sure on the best way to test this
void main() {
  group('$View', () {
    test('it returns a view file', () async {
      final view = TestView('foo', {
        'foo.mustache': 'bar',
      });

      expect(view.build()).toEqual('bar');
    });

    test('it returns a nested view file', () async {
      final view = TestView('foo.bar', {
        'foo/bar.mustache': 'bar',
      });

      expect(view.build()).toEqual('bar');
    });

    test('it replaces a value', () async {
      final view = TestView('foo', {
        'foo.mustache': 'bar {{ bar }}',
      }, {
        'bar': 'baz',
      });

      expect(view.build()).toEqual('bar baz');
    });

    test('it replaces a nested value', () async {
      final view = TestView('foo', {
        'foo.mustache': 'bar {{ bar.baz }}',
      }, {
        'bar': {
          'baz': 'foo',
        },
      });

      expect(view.build()).toEqual('bar foo');
    });

    test('it replaces a nested List value', () async {
      final view = TestView('foo', {
        'foo.mustache': 'bar {{ bar.baz }}',
      }, {
        'bar': {
          'baz': ['foo', 'bar'],
        },
      });

      expect(view.build()).toEqual("bar [foo, bar]");
    });

    test('it replaces a nested Map value', () async {
      final view = TestView('foo', {
        'foo.mustache': 'bar {{ bar.baz }}',
      }, {
        'bar': {
          'baz': {
            'foo': 'bar',
          },
        },
      });

      expect(view.build()).toEqual("bar {foo: bar}");
    });

    test('it ignores missing values', () async {
      final view = TestView('foo', {
        'foo.mustache': 'bar {{ bar }}',
      });

      expect(view.build()).toEqual('bar ');
    });

    test('it includes partials', () async {
      final view = TestView('foo', {
        'bar.mustache': 'bar',
        'foo.mustache': 'foo {{> bar }}',
      });

      expect(view.build()).toEqual('foo bar');
    });

    test('it includes nested partials', () async {
      final view = TestView('foo', {
        'bar/baz.mustache': 'bar',
        'foo.mustache': 'foo {{> bar.baz }}',
      });

      expect(view.build()).toEqual('foo bar');
    });

    test('it provides data to partials', () async {
      final view = TestView('foo', {
        'bar/baz.mustache': '{{ baz}}',
        'foo.mustache': 'foo {{> bar.baz }}',
      }, {
        'baz': 'bar',
      });

      expect(view.build()).toEqual('foo bar');
    });

    test('it extends but is not included if not yielded', () async {
      final view = TestView('foo', {
        'extended.mustache': 'extended',
        'foo.mustache': '{{> !extends extended }} foo',
      });

      expect(view.build()).toEqual('extended');
    });

    test('it extends & yields child content', () async {
      final view = TestView('foo', {
        'extended.mustache': 'extended -> {{> !yield }}',
        'foo.mustache': '{{> !extends extended }}foo',
      });

      expect(view.build()).toEqual('extended -> foo');
    });

    test('it recursivley extends', () async {
      final view = TestView('foo', {
        'foo.mustache': '{{> !extends bar }}foo',
        'bar.mustache': '{{> !extends baz }}bar -> {{> !yield }}',
        'baz.mustache': 'baz -> {{> !yield }}',
      });

      expect(view.build()).toEqual('baz -> bar -> foo');
    });

    test('it throws if a single view extends more than once', () async {
      final view = TestView('foo', {
        'foo.mustache': '{{> !extends bar }}{{> !extends bar }}foo',
        'bar.mustache': '{{> !extends baz }}bar -> {{> !yield }}',
      });

      expect(() => view.build()).throws.isException();
    });

    test('it escapes HTML', () async {
      final view = TestView('foo', {
        'foo.mustache': '{{ bar }}',
      }, {
        'bar': '<b>baz</b>'
      });

      expect(view.build()).toEqual('&lt;b&gt;baz&lt;&#x2F;b&gt;');
    });

    test('it un-escapes HTML with tripple brackets', () async {
      final view = TestView('foo', {
        'foo.mustache': '{{{ bar }}}',
      }, {
        'bar': '<b>baz</b>'
      });

      expect(view.build()).toEqual('<b>baz</b>');
    });

    test('it hides a section with a false value', () async {
      final view = TestView('foo', {
        'foo.mustache': 'foo {{#bar}}bar{{/bar}}',
      }, {
        'bar': false,
      });

      expect(view.build()).toEqual('foo ');
    });

    test('it returns a list of values', () async {
      final view = TestView('foo', {
        'foo.mustache': '{{#bar}}{{ baz }}{{/bar}}',
      }, {
        'bar': [
          {'baz': 'foo'},
          {'baz': 'bar'},
          {'baz': 'baz'},
          {'doh': 'nope'},
        ],
      });

      expect(view.build()).toEqual('foobarbaz');
    });
    test('it returns a list of values', () async {
      final view = TestView('foo', {
        'foo.mustache': '{{#bar}}{{ baz }}{{/bar}}',
      }, {
        'bar': [
          {'baz': 'foo'},
          {'baz': 'bar'},
          {'baz': 'baz'},
          {'doh': 'nope'},
        ],
      });

      expect(view.build()).toEqual('foobarbaz');
    });

    test('it returns an inverted list value', () async {
      final view = TestView('foo', {
        'foo.mustache': '{{#bar}}{{ baz }}{{/bar}}{{^bar}}inverted{{/bar}}',
      }, {
        'bar': [],
      });

      expect(view.build()).toEqual('inverted');
    });

    test('it ignores comments', () async {
      final view = TestView('foo', {
        'foo.mustache': 'foo{{! ignore me }}bar',
      }, {
        'bar': [],
      });

      expect(view.build()).toEqual('foobar');
    });
  });
}
